package MultiCastZoo;

import java.io.IOException;
import java.net.MulticastSocket;
import java.net.InetAddress;
import java.net.DatagramPacket;
import java.net.SocketException;
import java.net.SocketTimeoutException;

/**
 * Multicast Zoo - A vulnerable distributed zoo system
 * with protected command execution.
 * Written for a security exercise in
 * Computer Network Security  (ECE 6612 Spring 2008)
 * at Georgia Tech
 * @author David Bauer and Chris Lee
 *
 */
public class MultiCastZoo {
	static MulticastSocket sock;
	static InetAddress group;
	static int port = 446;

	static void sendMsg(String msg) {
		byte [] buf = msg.getBytes();
		DatagramPacket packet = new DatagramPacket(buf, 0, buf.length, group, port);
		System.out.println("Sending: \"" + msg + "\"");
		try {
			sock.send(packet);
		} catch(IOException e) {}
	}

	public static void main(String[] args) {
		int time = 30000;		// 30 seconds in milliseconds
		long endTime = System.currentTimeMillis() + time;
		byte[] buf = new byte[1472];
		try {
			Zoo z = new Zoo("zootest1.cfg");
			sock = new MulticastSocket(port);
			group = InetAddress.getByName("230.0.0.1");
			try {
				sock.joinGroup(group);
			} catch (SocketException e) {
				System.out.println("Caught exception while trying to join multicast group: " + e);
				System.out.println("Please check your network settings.  Multicast must be enabled and routable.");
				return;
			}
			DatagramPacket packet = new DatagramPacket(buf, buf.length);
			while(true) {
				long currentTime = System.currentTimeMillis();
				if (currentTime + time < endTime) endTime = currentTime + time;	// Problems?  Reset the time
				while (currentTime < endTime) {
					sock.setSoTimeout((int) (endTime - currentTime));
					try {
						sock.receive(packet);
						String received = new String(packet.getData(), packet.getOffset(), packet.getLength());
						System.out.println("Received: " + received);
						if(received.startsWith("CMD"))
							CommandParser.receiveMessage(received);
						else
							z.receiveMessage(received);
					} catch (SocketTimeoutException e) {
						break;
					}
					currentTime = System.currentTimeMillis();
				}
				endTime += time;
				String[] messages = z.performRound();
				for(String msg :messages) { 
					sendMsg(msg);
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
}